/***************************************************************************
 *   Copyright (C) 2007, IBM                                               *
 *                                                                         *
 *   Maintained By:                                                        *
 *   Eric Munson and Brad Peters                                           *
 *   munsone@us.ibm.com, bpeters@us.ibm.com                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the Lesser GNU General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or at your option) any later version.                        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU Lesser General Public License for more details.                   *
 *                                                                         *
 *   You should have received a copy of the Lesser GNU General Public      *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "libvpd-1/vpddbenv.h"
#include <stdio.h>

#define KEY_BUFFER_SIZE 256
#define DATA_BUFFER_SIZE 12288

struct vpddbenv * new_vpddbenv( const char *dir, const char *file,
                                u_int32_t flags )
{
	struct vpddbenv *ret = NULL;
	int rc = 0;
	
	ret = malloc( sizeof( struct vpddbenv ) );
	if( !ret )
		return ret;
	
	memset( ret, 0, sizeof( struct vpddbenv ) );
	
	if( dir == NULL || strcmp( dir, "" ) == 0 )
		strcpy( ret->envDir, DEFAULT_ENV );
	else
		strncpy( ret->envDir, dir, 256 );
	
	if( file == NULL || strcmp( file, "" ) )
		strcpy( ret->dbFileName, DEFAULT_DB );
	else
		strncpy( ret->dbFileName, file, 256 );
	
	ret->cFlags = flags;
	
	rc = db_env_create( &ret->env, 0 );
	if( rc )
		goto newerr;
	
	rc = ret->env->open( ret->env, ret->envDir,
			ret->cFlags | DB_INIT_MPOOL | DB_CREATE, 0 );
	if( rc )
		goto newerr;
	
	rc = db_create( &ret->db, ret->env, 0 );
	if( rc )
		goto newerr;
	
	rc = ret->db->set_bt_compare( ret->db, compare_keys );
	if( rc )
		goto newerr;
	
	rc = ret->db->open( ret->db, NULL, ret->dbFileName, NULL, DB_BTREE, 
			ret->cFlags | DB_CREATE, 0 );
	if( rc )
		goto newerr;
	
	return ret;
	
newerr:
	fprintf( stderr, "Berkeley db error '%s'\n", db_strerror( rc ) );
	free_vpddbenv( ret );
	return NULL;
}

void free_vpddbenv( struct vpddbenv *freeme )
{
	if( !freeme )
		return;
	
	if( freeme->db )
		freeme->db->close( freeme->db, 0 );
	
	if( freeme->env )
		freeme->env->close( freeme->env, 0 );
	
	free( freeme );
}

struct component* fetch_component( struct vpddbenv *db, const char *deviceID )
{
	char strkey[ KEY_BUFFER_SIZE ] = { 0 };
	char *buffer;
	struct component *ret;
	int rc;
	DBT key, data;

	buffer = (char*)malloc( DATA_BUFFER_SIZE );
	if( !buffer )
		return NULL;
	
	memset( buffer, 0, DATA_BUFFER_SIZE );
	memset( &data, 0, sizeof( DBT ) );
	memset( &key, 0, sizeof( DBT ) );

	strncpy( strkey, deviceID, KEY_BUFFER_SIZE - 1 );
	key.data = strkey;
	key.size = KEY_BUFFER_SIZE;

	data.data = buffer;
	data.ulen = DATA_BUFFER_SIZE;
	data.flags = DB_DBT_USERMEM;
	
	rc = db->db->get( db->db, NULL, &key, &data, 0 );
	if( rc )
	{
		fprintf( stderr, "Error fetching '%s': %s\n", deviceID, db_strerror( rc ) );
		free( buffer );
		return NULL;
	}

	ret = unpack_component( data.data );
	free( buffer );
	return ret;
}

struct system* fetch_system( struct vpddbenv *db )
{
	char strkey[ KEY_BUFFER_SIZE ] = { 0 };
	char *buffer;
	struct system *ret;
	int rc;
	DBT key, data;
	
	buffer = (char*)malloc( DATA_BUFFER_SIZE );
	if( !buffer )
		return NULL;
	
	memset( buffer, 0, DATA_BUFFER_SIZE );
	memset( &data, 0, sizeof( DBT ) );
	memset( &key, 0, sizeof( DBT ) );

	strncpy( strkey, SYS_ID, KEY_BUFFER_SIZE - 1 );
	key.data = strkey;
	key.size = KEY_BUFFER_SIZE;

	data.data = buffer;
	data.ulen = DATA_BUFFER_SIZE;
	data.flags = DB_DBT_USERMEM;
	
	rc = db->db->get( db->db, NULL, &key, &data, 0 );
	if( rc )
	{
		fprintf( stderr, "Error fetching '%s': %s\n", SYS_ID, db_strerror( rc ) );
		free( buffer );
		return NULL;
	}

	ret = unpack_system( data.data );
	free( buffer );
	return ret;
}

int compare_keys( DB *db, const DBT *key1, const DBT *key2 )
{
	return strcmp( (char*)key1->data, (char*)key2->data );
}
